#!/bin/ch

// this script will generate the ch package

#include <unistd.h>   // for access()
#include <dlfcn.h>    // for dlopen()
#include <chshell.h>  // for chinfo()

int addChdlHeader(char *headerfile, char *macro, char *chdlheader);

string_t pkgname="chunixodbc";          // define package name
string_t libname="sql";              // assume the lib is libsql.so 

string_t lib_header_dir;             // library header files directory 
string_t createchf_dir="chfcreate/"; // library directory containing chf files
string_t c_dir ="c/";                // directory  to generate .dl file
string_t include_dir="include/";     // header files used for Ch 
string_t demo_dir="demos/";          // directory  to generate .dl file
string_t cwd = _cwd;                 // the current working directory

chinfo_t info;

//make sure pgkcreate.ch is run from the current working directory
if(access("pkgcreate.ch", R_OK)) {
    echo Run ./pkgcreate.ch in the current directory.
    exit(-1);
}

// make sure you are running ch version workable with this script.
chinfo(&info);
if ((info.vermajor*100+ info.verminor*10 +  info.vermicro) < 501) {
   echo "To run this script, you need to install Ch version 5.0.1.12201 or higher"
   echo "You can download the latest version from http://www.softintegration.com/download"
   exit(-1);
}

/*  
    we assume the library header files are either 
    in /usr/local/include or /usr/include 
*/
if (access("/usr/local/include/sql.h", R_OK ) == 0)
{
	echo "we assume unixodbc header files are located in /usr/local/include";
	lib_header_dir="/usr/local/include";
}
else if  (access("/usr/include/sql.h", R_OK ) == 0)
{
	echo "we assume unixodbc header files are located in /usr/include";
	lib_header_dir = "/usr/include";
}
else
{
	echo "Error: unable to find iODBC header files.";
        echo "Please download iODBC source code and install it first.";
	exit(-1);
}

/* clean up existing chunixodbc directory  */
if (access(pkgname, R_OK) ==0)
{
        echo "Clean up package directory ${pacakge_name} now";
        rm -rf $pkgname;
}

// create the directory for distribution
mkdir $pkgname $pkgname/dl $pkgname/lib $pkgname/include $pkgname/demos

echo copy library header files for ch
cp $lib_header_dir/sqlext.h $include_dir 
cp $lib_header_dir/sqlucode.h $include_dir
cp $lib_header_dir/sqltypes.h $include_dir
cp $lib_header_dir/sql.h     $include_dir
cp $lib_header_dir/unixodbc_conf.h $include_dir
// copy the files which might not exist in unixodbc 2.3 or higher version 
// ignore the error
cp $lib_header_dir/unixodbc_conf_*.h $include_dir/unixodbc_conf.h >/dev/null 2>&1

// needs special handling for sql.h file used for Ch
char *macro ="#define __SQL_H";

string_t chdlheader =stradd(
  "\n\n",
  "#if defined(_CH_) \n"
  "#pragma package <", pkgname, "> \n",
  "#include <chdl.h> \n",
  "LOAD_CHDL(", libname, ")\n",
  "#endif \n");

// copy the library header file with the change
// it will add chdlheader under "#define _SQL_H"; in sql.h
addChdlHeader(stradd(include_dir, "/sql.h"), macro, chdlheader);

echo generate header files containing exported functions only
processhfile("*SQL_API", 1,  ";",   stradd(include_dir, "/sql.h"), stradd(createchf_dir, "sql.h"), NULL);

processhfile("SQLRETURN", 1,  ";",  stradd(include_dir, "/sqlext.h"), stradd(createchf_dir, "/sqlext.h"), NULL);
removeFuncProto(stradd(createchf_dir, "/sqlext.h"), "TraceReturn", 0);

processhfile("*SQL_API", 1,  ";",  stradd(include_dir, "/sqlucode.h"), stradd(createchf_dir, "/sqlucode.h"), NULL);

echo generating *.chf and *_chdl.c files

// the default handler for sql.h is _Chsql_handle
echo process sql.h
c2chf $createchf_dir/sql.h -i c sql.h -o c ${c_dir} -o chf ${pkgname}/lib
 
// for sqlext.h and sqlucode.h, they share the same handle as sql.h 
echo process sqlext.h
c2chf $createchf_dir/sqlext.h -h _Ch${libname}_handle -i c sqlext.h -o c ${c_dir} -o chf ${pkgname}/lib 

echo process sqlucode.h
c2chf $createchf_dir/sqlucode.h -h _Ch${libname}_handle -i c sql.h -i c sqlucode.h -o c ${c_dir} -o chf ${pkgname}/lib 

cd ${c_dir}
echo clean up ${c_dir} object files
make clean
make

echo test if .dl file is created successfully
string_t libsqldl = stradd(`pwd`, "/libsql.dl");

if ( strcmp(`uname`,"Linux") ==0 )
{
    chcon -t texrel_shlib_t $libsqldl  >/dev/null 2>&1
}

if ( dlopen(libsqldl, 1) == NULL)
{
    echo "error in libsql.dl";
    printf("%s \n", dlerror());
    exit(-1);
}

// go back to previous directory
cd $cwd

echo "libsql.dl is successfully created";

echo copy the created files
cp $c_dir/lib${libname}.dl $pkgname/dl/
cp $include_dir/*.h $pkgname/include/
cp -p $demo_dir/* $pkgname/demos/

echo "${pkgname} is successfully created";

/*******************************************************************
 * * This function will add the required preprocessing directives
 * * needed by Ch to a header file.
 * * headerfile:   header file to be changed
 * * macro:        token to be recoganized for insertion of chdlheader
 * * chdlheader:   preprocessing directives to be inserted
 * *******************************************************************/
int addChdlHeader(char *headerfile, char *macro, char *chdlheader)
{
   char *stop_ptr;
   string_t tempfile, token;
   string_t result;
			    
   tempfile = ``cat $headerfile``;
   token = strstr( tempfile, macro);
   stop_ptr = strstr(token, "\n");
   strncpy(result, tempfile, strlen(tempfile)-strlen(stop_ptr));
   result=stradd(result, chdlheader);
   result=stradd(result, stop_ptr);
   
   FILE *stream;
   if(!(stream = fopen("_tmpfile", "w")))
   {
      perror("_tmpfile");
      return -1;
   }
   fprintf(stream, "%s", result);
   fclose(stream);
   mv -f _tmpfile $(headerfile)

   return 0;

}
